@@ -1,5 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
+ * Copyright © 2014 Pranav Kant
  * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
@@ -48,6 +49,7 @@
 #include "photos-source-manager.h"
 #include "photos-tracker-overview-controller.h"
 #include "photos-view-container.h"
+#include "photos-view-model.h"
 
 
 typedef struct _PhotosEmbedSearchState PhotosEmbedSearchState;
@@ -85,7 +87,6 @@ struct _PhotosEmbedPrivate
   PhotosModeController *mode_cntrlr;
   PhotosSearchController *srch_cntrlr;
   PhotosTrackerController *trk_ovrvw_cntrlr;
-  PhotosWindowMode old_mode;
   guint load_show_id;
 };
 
@@ -142,6 +143,65 @@ photos_embed_clear_search (PhotosEmbed *self)
 }
 
 
+static GtkWidget*
+photos_embed_get_view_container_from_mode (PhotosEmbed *self, PhotosWindowMode mode)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+  GtkWidget *view_container;
+
+  switch (mode)
+    {
+    case PHOTOS_WINDOW_MODE_COLLECTIONS:
+      view_container = priv->collections;
+      break;
+
+    case PHOTOS_WINDOW_MODE_FAVORITES:
+      view_container = priv->favorites;
+      break;
+
+    case PHOTOS_WINDOW_MODE_OVERVIEW:
+      view_container = priv->overview;
+      break;
+
+
+    case PHOTOS_WINDOW_MODE_SEARCH:
+      view_container = priv->search;
+      break;
+
+    default:
+      g_assert_not_reached ();
+    }
+
+  return view_container;
+}
+
+
+static void
+photos_embed_activate_result (PhotosEmbed *self)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+  GtkTreeIter iter;
+  GtkListStore *store;
+  GtkWidget *view_container;
+  PhotosWindowMode mode;
+
+  mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
+  view_container = photos_embed_get_view_container_from_mode (self, mode);
+  store = photos_view_container_get_model (PHOTOS_VIEW_CONTAINER (view_container));
+
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter))
+    {
+      GObject *item;
+      gchar *id;
+
+      gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, PHOTOS_VIEW_MODEL_URN, &id, -1);
+      item = photos_base_manager_get_object_by_id (priv->item_mngr, id);
+      photos_base_manager_set_active_object (priv->item_mngr, item);
+      g_free (id);
+    }
+}
+
+
 static void
 photos_embed_prepare_for_preview (PhotosEmbed *self)
 {
@@ -164,7 +224,6 @@ photos_embed_item_load (GObject *source_object, GAsyncResult *res, gpointer user
   GeglNode *node;
   GtkListStore *model;
   GtkTreePath *current_path;
-  GtkWidget *view_container;
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
   PhotosWindowMode mode;
 
@@ -176,37 +235,19 @@ photos_embed_item_load (GObject *source_object, GAsyncResult *res, gpointer user
     goto out;
 
   mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
-  switch (mode)
-    {
-    case PHOTOS_WINDOW_MODE_COLLECTIONS:
-      view_container = priv->collections;
-      break;
 
-    case PHOTOS_WINDOW_MODE_FAVORITES:
-      view_container = priv->favorites;
-      break;
-
-    case PHOTOS_WINDOW_MODE_OVERVIEW:
-      view_container = priv->overview;
-      break;
-
-    case PHOTOS_WINDOW_MODE_PREVIEW:
-      view_container = NULL;
-      break;
-
-    case PHOTOS_WINDOW_MODE_SEARCH:
-      view_container = priv->search;
-      break;
-
-    default:
-      g_assert_not_reached ();
-    }
-
-  /* If we are already in the preview and navigating using the
-   * buttons, then we don't need this.
+  /* This is not needed when:
+   *  - activated from the search provider
+   *  - already in the preview and navigating using the buttons
    */
-  if (mode != PHOTOS_WINDOW_MODE_PREVIEW)
+  if (mode == PHOTOS_WINDOW_MODE_COLLECTIONS
+      || mode == PHOTOS_WINDOW_MODE_FAVORITES
+      || mode == PHOTOS_WINDOW_MODE_OVERVIEW
+      || mode == PHOTOS_WINDOW_MODE_SEARCH)
     {
+      GtkWidget *view_container;
+
+      view_container = photos_embed_get_view_container_from_mode (self, mode);
       current_path = photos_view_container_get_current_path (PHOTOS_VIEW_CONTAINER (view_container));
       model = photos_view_container_get_model (PHOTOS_VIEW_CONTAINER (view_container));
       photos_preview_view_set_model (PHOTOS_PREVIEW_VIEW (priv->preview), GTK_TREE_MODEL (model), current_path);
@@ -521,9 +562,6 @@ photos_embed_search_changed (PhotosEmbed *self)
   if (mode == PHOTOS_WINDOW_MODE_PREVIEW)
     return;
 
-  if (priv->old_mode == PHOTOS_WINDOW_MODE_PREVIEW)
-    return;
-
   object = photos_base_manager_get_active_object (priv->src_mngr);
   source_id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
 
@@ -535,11 +573,9 @@ photos_embed_search_changed (PhotosEmbed *self)
   if (g_strcmp0 (search_type_id, PHOTOS_SEARCH_TYPE_STOCK_ALL) == 0
       && g_strcmp0 (source_id, PHOTOS_SOURCE_STOCK_ALL) == 0
       && (str == NULL || str [0] == '\0'))
-    mode = priv->old_mode;
+    photos_mode_controller_go_back (priv->mode_cntrlr);
   else
-    mode = PHOTOS_WINDOW_MODE_SEARCH;
-
-  photos_mode_controller_set_window_mode (priv->mode_cntrlr, mode);
+    photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_SEARCH);
 }
 
 
@@ -551,8 +587,6 @@ photos_embed_window_mode_changed (PhotosModeController *mode_cntrlr,
 {
   PhotosEmbed *self = PHOTOS_EMBED (user_data);
 
-  self->priv->old_mode = old_mode;
-
   if (mode == PHOTOS_WINDOW_MODE_COLLECTIONS)
     photos_embed_prepare_for_collections (self);
   else if (mode == PHOTOS_WINDOW_MODE_FAVORITES)
@@ -612,6 +646,7 @@ photos_embed_init (PhotosEmbed *self)
 {
   PhotosEmbedPrivate *priv;
   GApplication *app;
+  PhotosSearchbar *searchbar;
   PhotosSearchContextState *state;
   gboolean querying;
 
@@ -642,6 +677,8 @@ photos_embed_init (PhotosEmbed *self)
 
   priv->toolbar = photos_main_toolbar_new (GTK_OVERLAY (priv->stack_overlay));
   photos_main_toolbar_set_stack (PHOTOS_MAIN_TOOLBAR (priv->toolbar), GTK_STACK (priv->stack));
+  searchbar = photos_main_toolbar_get_searchbar (PHOTOS_MAIN_TOOLBAR (priv->toolbar));
+  g_signal_connect_swapped (searchbar, "activate-result", G_CALLBACK (photos_embed_activate_result), self);
 
   priv->ntfctn_mngr = g_object_ref_sink (photos_notification_manager_dup_singleton ());
   gtk_overlay_add_overlay (GTK_OVERLAY (priv->stack_overlay), priv->ntfctn_mngr);
@@ -674,7 +711,6 @@ photos_embed_init (PhotosEmbed *self)
                            self, G_CONNECT_SWAPPED);
 
   priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
-  priv->old_mode = PHOTOS_WINDOW_MODE_NONE;
   g_signal_connect (priv->mode_cntrlr,
                     "window-mode-changed",
                     G_CALLBACK (photos_embed_window_mode_changed),
