@@ -28,14 +28,12 @@
 #include <gio/gio.h>
 #include <libgd/gd.h>
 
-#include "photos-application.h"
-#include "photos-collection-manager.h"
+#include "photos-base-manager.h"
+#include "photos-filterable.h"
 #include "photos-overview-searchbar.h"
+#include "photos-search-context.h"
 #include "photos-search-controller.h"
-#include "photos-search-match-manager.h"
 #include "photos-search-type.h"
-#include "photos-search-type-manager.h"
-#include "photos-source-manager.h"
 
 
 struct _PhotosOverviewSearchbarPrivate
@@ -72,11 +70,11 @@ photos_overview_searchbar_active_changed (PhotosOverviewSearchbar *self,
 {
   PhotosOverviewSearchbarPrivate *priv = self->priv;
   GObject *object;
-  gchar *id;
+  const gchar *id;
   gchar *name;
 
   object = photos_base_manager_get_active_object (mngr);
-  g_object_get (object, "id", &id, NULL);
+  id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
   g_object_get (object, "name", &name, NULL);
 
   if (g_strcmp0 (id, "all") == 0)
@@ -88,7 +86,6 @@ photos_overview_searchbar_active_changed (PhotosOverviewSearchbar *self,
     }
 
   g_free (name);
-  g_free (id);
 }
 
 
@@ -97,20 +94,18 @@ photos_overview_searchbar_collection_active_changed (PhotosOverviewSearchbar *se
 {
   PhotosOverviewSearchbarPrivate *priv = self->priv;
   PhotosSearchType *search_type;
+  const gchar *id;
   const gchar *str;
-  gchar *id;
 
   search_type = PHOTOS_SEARCH_TYPE (photos_base_manager_get_active_object (priv->srch_typ_mngr));
   str = photos_search_controller_get_string (priv->srch_cntrlr);
-  g_object_get (search_type, "id", &id, NULL);
+  id = photos_filterable_get_id (PHOTOS_FILTERABLE (search_type));
 
   if (g_strcmp0 (str, "") != 0 || g_strcmp0 (id, "all") != 0)
     {
       photos_base_manager_set_active_object_by_id (priv->srch_typ_mngr, "all");
       gtk_entry_set_text (GTK_ENTRY (priv->search_entry), "");
     }
-
-  g_free (id);
 }
 
 
@@ -347,44 +342,46 @@ static void
 photos_overview_searchbar_init (PhotosOverviewSearchbar *self)
 {
   PhotosOverviewSearchbarPrivate *priv;
-  GtkApplication *app;
+  GApplication *app;
+  PhotosSearchContextState *state;
 
   self->priv = photos_overview_searchbar_get_instance_private (self);
   priv = self->priv;
 
-  app = photos_application_new ();
-  priv->select_all = g_action_map_lookup_action (G_ACTION_MAP (app), "select-all");
-  g_object_unref (app);
+  app = g_application_get_default ();
+  state = photos_search_context_get_state (PHOTOS_SEARCH_CONTEXT (app));
 
-  priv->src_mngr = photos_source_manager_dup_singleton ();
+  priv->select_all = g_action_map_lookup_action (G_ACTION_MAP (app), "select-all");
+
+  priv->src_mngr = g_object_ref (state->src_mngr);
   g_signal_connect_object (priv->src_mngr,
                            "active-changed",
                            G_CALLBACK (photos_overview_searchbar_source_active_changed),
                            self,
                            G_CONNECT_SWAPPED);
 
-  priv->srch_mtch_mngr = photos_search_match_manager_dup_singleton ();
+  priv->srch_mtch_mngr = g_object_ref (state->srch_mtch_mngr);
   g_signal_connect_object (priv->srch_mtch_mngr,
                            "active-changed",
                            G_CALLBACK (photos_overview_searchbar_search_match_active_changed),
                            self,
                            G_CONNECT_SWAPPED);
 
-  priv->srch_typ_mngr = photos_search_type_manager_dup_singleton ();
+  priv->srch_typ_mngr = g_object_ref (state->srch_typ_mngr);
   g_signal_connect_object (priv->srch_typ_mngr,
                            "active-changed",
                            G_CALLBACK (photos_overview_searchbar_search_type_active_changed),
                            self,
                            G_CONNECT_SWAPPED);
 
-  priv->col_mngr = photos_collection_manager_dup_singleton ();
+  priv->col_mngr = g_object_ref (state->col_mngr);
   g_signal_connect_object (priv->col_mngr,
                            "active-changed",
                            G_CALLBACK (photos_overview_searchbar_collection_active_changed),
                            self,
                            G_CONNECT_SWAPPED);
 
-  priv->srch_cntrlr = photos_search_controller_dup_singleton ();
+  priv->srch_cntrlr = g_object_ref (state->srch_cntrlr);
 }
 
 
